// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.io.IOException;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.FlatFileDAOFactory;

public class Checker {

    /**
     * @param args
     */
    public static void main(String[] args) {
        String date = "03/12/2013";
        String version = "1.5";
        String welcome = "This is the ROADEF/EURO Challenge 2014 checker, version " + version
                + " of " + date + ".";
        System.out.println(welcome);

        if (args.length != 2) {
            printUsage();
            System.exit(1);
        }

        ConstraintComposite constraint = new ConstraintComposite();
        CostComposite f1 = new CostComposite();
        CostComposite f2 = new CostComposite();
        CostComposite f3 = new CostComposite();
        float score1;
        float score2;
        float score3;
        boolean feasible;


        Logger logger = Logger.getLogger("logger");
        try {

            try {
                FileHandler fh = new FileHandler("log.txt");
                fh.setFormatter(new SimpleFormatter());
                logger.addHandler(fh);
                logger.info(welcome);
            } catch (SecurityException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            }

            DAOFactory factory = new FlatFileDAOFactory(args[0], args[1]);
            Datetime.MAX.setDay(
                    factory.getParametersDAO().getParameters().getNbDays());
            constraint.add(new SolutionConstraint(factory));
            constraint.add(new EntranceConstraint(factory));
            constraint.add(new ExitConstraint(factory));
            constraint.add(new ResourceUseConstraint(factory));
            constraint.add(new TransitionConstraint(factory));
            constraint.add(new MaintenanceOperationsConstraint(factory));
            constraint.add(new JunctionDisjunctionOperationsConstraint(
                    factory));
            constraint.add(new TrackGroupUseConstraint(factory));
            constraint.add(new MinimumReverseTimeConstraint(factory));
            constraint.add(new AtMostOneTrainPerDepartureConstraint(factory));
            constraint.add(new DepartureRequiredDBMConstraint(factory));
            constraint.add(new DepartureRequiredTBMConstraint(factory));
            constraint.add(new DepartureTrainCategoryCompatibilityConstraint(
                    factory));
            constraint.add(new AssembledTrainCategoryCompatibilityConstraint(
                    factory));
            constraint.add(new ResourceTrainCategoryCompatibilityConstraint(
                    factory));
            constraint.add(new PlatformUseMaximumDurationConstraint(factory));
            constraint.add(new ResourceUseMinimumDurationConstraint(factory));
            constraint.add(new ImposedConsumptionConstraint(factory));
            constraint.add(new TrackLengthConstraint(factory));
            constraint.add(new SingleTrackCapacityConstraint(factory));
            constraint.add(new MaintenanceResourcesCapacityConstraint(factory));
            constraint.add(new TrainOrderOnIndividualTrackConstraint(factory));
            constraint.add(new TrainOrderAfterJunctionConstraint(factory));
            constraint.add(new ExitFromResourceAfterDisjunctionConstraint(factory));
            constraint.add(new AssembledTrainsSimultaneousEventConstraint(
                    factory));
            constraint.add(new DepartureOrderConstraint(factory));
            feasible = constraint.validate();

            f1.add(new UncoveredDeparturesCost(factory));
            score1 = f1.calculate();

            f2.add(new TrackGroupConflictCost(factory));
            f2.add(new YardOverloadCost(factory));
            score2 = f2.calculate();

            f3.add(new OverMaintenanceCost(factory));
            f3.add(new JunctionDisjunctionCost(factory));
            f3.add(new PlatformUsageCost(factory));
            f3.add(new PreferredPlatformAssignementCost(factory));
            f3.add(new TrainReuseCost(factory));
            score3 = f3.calculate();

            System.out.println("Feasible : " + (feasible ? "True" : "False"));
            System.out.println("Score 1 : " + score1);
            System.out.println("Score 2 : " + score2);
            System.out.println("Score 3 : " + score3);
        } catch (ROADEFException e) {
            logger.log(Level.SEVERE, e.getError());
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private static void printUsage() {
        System.out.println("Usage: java -jar checker.jar path_to_instance/myInstance.zip path_to_solution/mySolution.csv");
    }

}
