// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Parameters;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.FillDAO;

/**
 * Train junction / disjunction operation cost.
 *
 * @author YDS
 *
 */
public class JunctionDisjunctionCost implements Cost {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public JunctionDisjunctionCost(final DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public final float calculate() throws ROADEFException {
        FillDAO fillDAO = factory.getFillDAO();
        Parameters parameters = factory.getParametersDAO().getParameters();
        float score = 0;
        Logger logger = Logger.getLogger("logger");

        for (Fill fill : fillDAO.loadAll()) {
            if (fill.getEvent().equals(Event.BegJunction)) {
                logger.log(Level.INFO, "Junction disjunction cost : "
                        + fill.getTrain().getName() + " has a junction at "
                        + fill.getDate().toString()
                        + ".");
                score += parameters.getJunCost();
            } else if (fill.getEvent().equals(Event.BegDisjunction)) {
                logger.log(Level.INFO, "Junction disjunction cost : "
                        + fill.getTrain().getName() + " has a disjunction at "
                        + fill.getDate().toString()
                        + ".");
                score += parameters.getDisjCost();
            }
        }
        return score;
    }

}
