// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.TrainDAO;

public class ResourceUseConstraint implements Constraint {

    private DAOFactory factory;

    public ResourceUseConstraint(DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        TrainDAO dao = factory.getTrainDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = dao.loadAll();
        for (Train train : trainList) {
            Fill previousEntry = null;
            for (Fill fill : train.getFill()) {
                if (previousEntry != null) {
                    if (previousEntry.getResource() != fill.getResource()) {
                        logger.log(Level.INFO, "Use of a ressource error : "
                            + train.getName() + " event was espected on "
                            + previousEntry.getResource().getName() + " ("
                            + fill.getResource().getName() + " at "
                            + fill.getDate().toString() + " found).");
                        valid = false;
                    }
                }
                if (fill.getEvent().equals(Event.EnterResource)) {
                    if (previousEntry != null) {
                        logger.log(Level.INFO, "Use of a ressource error : "
                            + train.getName() + " ExitResource event expected"
                            + " after EnterResource event at "
                            + previousEntry.getDate().toString() + ".");
                        valid = false;
                    }
                    previousEntry = fill;
                } else if (fill.getEvent().equals(Event.ExitResource)) {
                    if (previousEntry == null) {
                        logger.log(Level.INFO, "Use of a ressource error : "
                            + train.getName() + " EnterResource event expected"
                            + " before ExitResource event at "
                            + fill.getDate().toString() + ".");
                        valid = false;
                    }
                    previousEntry = null;
                }

            }
        }
        return valid;
    }

}
