package com.sncf.checker;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.TrainDAO;

/**
 * General solution errors.
 *
 * @author YDS
 *
 */
public class SolutionConstraint implements Constraint {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public SolutionConstraint(DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public boolean validate() throws ROADEFException {
        TrainDAO dao = factory.getTrainDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = dao.loadAll();
        for (Train train : trainList) {
            Fill prev = null;
            for (Fill fill : train.getFill()) {
                if (prev != null) {
                    if (fill.getDate().getTime() < prev.getDate().getTime()) {
                        logger.log(Level.INFO, "Error : "
                                + train.getName() + " events are not sorted by "
                                + "transition time (" + fill.getDate().toString()
                                + " event follows " + prev.getDate().toString()
                                + " event)");
                        valid = false;
                    }
                }
                if (fill.getDate().getTime() > Datetime.MAX.getTime()
                        || fill.getDate().getTime() < Datetime.MIN.getTime()) {
                    logger.log(Level.INFO, "Error : "
                            + train.getName() + " has an event outside "
                            + "the planning horizon (" + fill.getDate().toString()
                            + ")");
                    valid = false;
                }
                prev = fill;
            }
        }
        return valid;
    }

}
