// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.IndividualTrack;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.IndividualTrackDAO;

public class TrackLengthConstraint implements Constraint {

    private DAOFactory factory;

    public TrackLengthConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        IndividualTrackDAO individualTrackDAO = factory.getIndividualTrackDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<IndividualTrack> individualTrackList = individualTrackDAO.loadAll();
        for (IndividualTrack individualTrack : individualTrackList) {
            int length = 0;
            List<Fill> fillList = individualTrack.getFill();
            Collections.sort(fillList, new SortFillByDate());
            Datetime prev = new Datetime();
            for (Fill fill : fillList) {
                if (prev.getTime() != fill.getDate().getTime()) {
                    if (length > individualTrack.getLength())  {
                        logger.log(Level.INFO, "Track length error : "
                                + individualTrack.getName() + " total length of trains ("
                                + length + "m) exceeds its length ("
                                + individualTrack.getLength() + "m) at "
                                + prev.toString() + ".");
                        valid = false;
                    }
                }
                if (fill.getEvent().equals(Event.EnterResource)) {
                    length = length + fill.getTrain().getCategory().getLength();
                } else if (fill.getEvent().equals(Event.ExitResource)) {
                    length = length - fill.getTrain().getCategory().getLength();
                }
                prev = fill.getDate();
            }

            if (length > individualTrack.getLength())  {
                logger.log(Level.INFO, "Track length error : "
                        + individualTrack.getName() + " total length of trains ("
                        + length + ") exceeds its length ("
                        + individualTrack.getLength() + ") at "
                        + prev.toString() + ".");
                valid = false;
            }
        }
        return valid;
    }

}
