// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.ImposedConsumption;
import com.sncf.checker.bo.Yard;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.ImposedConsumptionDAO;
import com.sncf.checker.io.YardDAO;

public class YardOverloadCost implements Cost {

    private DAOFactory factory;

    public YardOverloadCost(DAOFactory factory) {
        this.factory = factory;
    }

    public float calculate() throws ROADEFException {
        float score = 0;
        YardDAO yardDAO = factory.getYardDAO();
        ImposedConsumptionDAO consumptionDAO = factory.getImposedConsumptionDAO();
        Logger logger = Logger.getLogger("logger");

        List<ImposedConsumption> consumptionList = consumptionDAO.loadAll();

        for (Yard yard : yardDAO.loadAll()) {
            ArrayList<YardEvent> eventList = new ArrayList<YardEvent>();
            for (Fill fill : yard.getFill()) {
                if (fill.getEvent().equals(Event.EnterResource)) {
                    eventList.add(new YardEvent(fill.getDate(), 1));
                } else if (fill.getEvent().equals(Event.ExitResource)) {
                    eventList.add(new YardEvent(fill.getDate(), -1));
                }
            }

            for (ImposedConsumption consumption : consumptionList) {
                if (consumption.getResource() == yard) {
                    eventList.add(new YardEvent(consumption.getBeg(), consumption.getNb()));
                    eventList.add(new YardEvent(consumption.getEnd(), -consumption.getNb()));
                }
            }

            Collections.sort(eventList);
            int nbTrain = 0;
            for (YardEvent event : eventList) {
                nbTrain += event.getBalance();
                if (nbTrain > yard.getCapacity() && event.getBalance() > 0) {
                    logger.log(Level.INFO, "Yard overload cost : "
                            + "at " + event.getDate().toString()
                            + " there are " + nbTrain + " trains.");
                    score++;
                }
            }
        }

        return score;
    }

    private class YardEvent implements Comparable<YardEvent> {

        public YardEvent(Datetime date, int balance) {
            this.date = date;
            this.balance = balance;
        }

        private Datetime date;

        public Datetime getDate() {
            return date;
        }

        private int balance;

        public int getBalance() {
            return balance;
        }

        public int compareTo(YardEvent arg0) {
            if (date.getTime() < arg0.date.getTime()) {
                return -1;
            } else if (date.getTime() > arg0.date.getTime()) {
                return 1;
            } else {
                if (balance < arg0.balance) {
                    return -1;
                } else if (balance > arg0.balance) {
                    return 1;
                } else {
                    return 0;
                }
            }

        }

    }
}
