// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.bo;

import java.util.ArrayList;
import java.util.List;

import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.FillDAO;
import com.sncf.checker.io.GateDAO;
import com.sncf.checker.io.TrainCategoryDAO;

/**
 * Resource.
 *
 * @author YDS
 *
 */
public abstract class Resource {

    private GateDAO gateDAO;
    private FillDAO fillDAO;
    private TrainCategoryDAO categoryDAO;

    /**
     * Constructor.
     */
    public Resource() {
        gateList = new ArrayList<Gate>();
        fillList = new ArrayList<Fill>();
        compCatRes = new ArrayList<TrainCategory>();
    }

    /**
     * lazy-loading version constructor.
     *
     * @param occupationDAO
     * @param fillDAO
     */
    public Resource(final GateDAO gateDAO, final FillDAO fillDAO, TrainCategoryDAO categoryDAO) {
        this.fillDAO = fillDAO;
        this.gateDAO = gateDAO;
        this.categoryDAO = categoryDAO;
    }

    /** name. */
    private String name;

    /**
     * Get the name.
     *
     * @return name
     */
    public final String getName() {
        return name;
    }

    /**
     * Set the name.
     *
     * @param name String to set
     */
    public final void setName(final String name) {
        this.name = name;
    }

    /** parking possibility. */
    private boolean parking;

    /**
     * Information about parking possibility.
     *
     * @return true if parking is possible, false otherwise
     */
    public final boolean isParking() {
        return parking;
    }

    /**
     * Set whether or not the occupation can be used for parking.
     *
     * @param parking boolean true if parking is possible, false otherwise
     */
    public final void setParking(final boolean parking) {
        this.parking = parking;
    }

    /** Fill object list. */
    private ArrayList<Gate> gateList;

    /**
     * Get all incoming occupations.
     *
     * @return Fill object list
     */
    public final List<Gate> getGates() {
        if (gateList == null) {
            gateList = new ArrayList<Gate>(gateDAO.loadNeighborsOf(name));
        }
        return gateList;
    }

    /**
     * Add an occupation at given time.
     *
     * @param fill
     */
    public final void addGate(final Gate gate) {
        if (gateList == null) {
            gateList = new ArrayList<Gate>(gateDAO.loadNeighborsOf(name));
        }
        gateList.add(gate);
    }

    /** Fill object list. */
    private ArrayList<Fill> fillList;

    /**
     * Get all incoming occupations.
     *
     * @return Fill object list
     * @throws ROADEFException
     */
    public final List<Fill> getFill() throws ROADEFException {
        if (fillList == null) {
            fillList = new ArrayList<Fill>(fillDAO.loadByOccupation(name));
        }
        return fillList;
    }

    /**
     * Add an occupation at given time.
     *
     * @param fill
     * @throws ROADEFException
     */
    public final void addFill(final Fill fill) throws ROADEFException {
        if (fillList == null) {
            fillList = new ArrayList<Fill>(fillDAO.loadByOccupation(name));
        }
        fillList.add(fill);
    }

    /** Fill object list. */
    private ArrayList<TrainCategory> compCatRes;

    /**
     * Get all incoming occupations.
     *
     * @return Fill object list
     */
    public final List<TrainCategory> getCompCatRes() {
        if (compCatRes == null) {
            compCatRes = new ArrayList<TrainCategory>(categoryDAO.loadByResource(name));
        }
        return compCatRes;
    }

    /**
     * Add an occupation at given time.
     *
     * @param fill
     */
    public final void addCompCatRes(final TrainCategory category) {
        if (compCatRes == null) {
            compCatRes = new ArrayList<TrainCategory>(categoryDAO.loadByResource(name));
        }
        compCatRes.add(category);
    }
}
