// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.ArrDepSequence;
import com.sncf.checker.common.TreeMultiMap;

/**
 * The data source is a tabulation separated file containing the ArrDepSequence objects.
 *
 * @author YDS
 *
 */
public class ArrDepSequenceFlatFileDAO extends FlatFileDAO implements ArrDepSequenceDAO {

    /** scenario file path. */
    private final String scenario;
    /** ArrDepSequence indexed by name. */
    private TreeMultiMap<String, ArrDepSequence> arrdepIndexedCache = null;

    public ArrDepSequenceFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /** TrackGroup DAO. */
    private TrackGroupDAO trackGroupDAO;

    /**
     * TrackGroup DAO setter.
     *
     * @param dao TrackGroup DAO
     */
    public void setTrackGroupDAO(final TrackGroupDAO dao) {
        trackGroupDAO = dao;
    }

    public List<ArrDepSequence> loadByArrDep(String name) {
        initialize();
        return arrdepIndexedCache.get(name);
    }

    /**
     * Extract all trains from arrDepSequences.csv in zip file.
     */
    private void initialize() {
        if (arrdepIndexedCache == null) {
            arrdepIndexedCache = new TreeMultiMap<String, ArrDepSequence>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "arrDepSequences.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));
                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    ArrDepSequence temp;

                    temp = deserialize(splits);
                    arrdepIndexedCache.put(splits[0], temp);
                    temp.setTrackGroup(trackGroupDAO.loadByName(splits[1]));
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a ArrDepSequence object.
     *
     * @param entry data extracted from the file
     * @return ArrDepSequence
     */
    private ArrDepSequence deserialize(final String[] raw) {
        ArrDepSequence entity = new ArrDepSequence();

        entity.setOrderNumber(Integer.parseInt(raw[2]));

        return entity;
    }

}
