// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.Gate;
import com.sncf.checker.common.TreeMultiMap;

public class GateFlatFileDAO extends FlatFileDAO implements GateDAO {

    /** scenario file path. */
    private final String scenario;
    private TreeMultiMap<String, Gate> neighborsCache = null;

    /**
     * Constructor.
     *
     * @param scenario file path to the scenario zip archive
     * @param solution file path to the solution file
     */
    public GateFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /** Resource DAO. */
    private ResourceDAO resourceDAO;

    /**
     * Resource DAO setter.
     *
     * @param dao Resource DAO
     */
    public void setResourceDAO(final ResourceDAO dao) {
        this.resourceDAO = dao;
    }

    /**
     * Load a platform by name.
     *
     * @param name the searched platform name
     * @return platform, null if not found
     */
    public List<Gate> loadNeighborsOf(final String name) {
        initialize();
        ArrayList<Gate> list = new ArrayList<Gate>();
        list.addAll(neighborsCache.get(name));
        return list;
    }

    /**
     * Extract all platforms from gates.csv in zip file. Extract compatible platforms from CompDutPlat.txt.
     */
    private void initialize() {
        if (neighborsCache == null) {
            neighborsCache = new TreeMultiMap<String, Gate>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "gates.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";", -1);
                    Gate temp;

                    temp = deserialize(splits);
                    neighborsCache.put(splits[0], temp);
                    temp.setNeighborResource(resourceDAO.loadByName(splits[3]));
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a Platform object.
     *
     * @param entry data extracted from the file
     * @return platform
     */
    private Gate deserialize(final String[] raw) {
        Gate entity = new Gate();

        entity.setSide(raw[1]);
        entity.setIndex(Integer.parseInt(raw[2]));
        if (!raw[3].equals("")) {
            entity.setNeighborSide(raw[4]);
            entity.setNeighborIndex(Integer.parseInt(raw[5]));
        }

        return entity;
    }

}
